/*************************************************************************************************
Dmitri Korotkevitch
http://aboutsqlserver.com
email: dmitri@aboutsqlserver.com

24HOP Russia (2015) - Size does Matter
05.Analyzing and Reducing Internal Fragmentation
*************************************************************************************************/
use [24HOP_2015]
go

select 
    index_id, partition_number, alloc_unit_type_desc
    ,index_level, page_count, page_count * 8 / 1024 as [Size MB]
	, avg_page_space_used_in_percent
from 
    sys.dm_db_index_physical_stats
    (
        db_id() /*Database */
        ,object_id(N'dbo.Customers') /* Table (Object_ID) */
        ,1 /* Index ID */
        ,null /* Partition ID  NULL  all partitions */
        ,'detailed' /* Mode */
    )
go

-- Size of the Clustered Index:
-- Number of Pages:
-- Logical Reads: 
set statistics io on
select max(CreditLimit) from dbo.Customers option (maxdop 1)
set statistics io off
go

alter index IDX_Customers_CustomerID on dbo.Customers rebuild with (fillfactor=100)
go

select 
    index_id, partition_number, alloc_unit_type_desc
    ,index_level, page_count, page_count * 8 / 1024 as [Size MB]
	, avg_page_space_used_in_percent
from 
    sys.dm_db_index_physical_stats
    (
        db_id() /*Database */
        ,object_id(N'dbo.Customers') /* Table (Object_ID) */
        ,1 /* Index ID */
        ,null /* Partition ID  NULL  all partitions */
        ,'detailed' /* Mode */
    )
go

if object_id('tempdb..#TempUID') is not null drop table #TempUID
create table #TempUID(ID uniqueidentifier default NEWID(), Num int)
go

declare
	@I int = 21
	,@X xml

select @X = 
	(select * from master.sys.objects for xml raw, root('Data')) 

truncate table #TempUID
	 
;with n1(c) as (select 0 union all select 0) -- 2 rows
,n2(c) as (select 0 from n1 as t1 cross join n1 as t2) -- 4 rows
,n3(c) as (select 0 from n2 as t1 cross join n2 as t2) -- 16 rows
,n4(c) as (select 0 from n3 as t1 cross join n3 as t2) -- 256 rows
,n5(c) as (select 0 from n4 as t1 cross join n4 as t2) -- 65,536 rows
,ids(id) as (select row_number() over (order by (select null)) from n5)
	insert into #TempUID(Num) 
		select ID from Ids

insert into dbo.Customers(CustomerId, CustomerNumber, CustomerName, CreditLimit, Attributes)
	select ID 
		,convert(nvarchar(16),@I * 65536 + 1 + Num)
		,N'Customer ' + convert(nvarchar(48),ID)
		,Abs(CheckSum(ID))
		,iif(Num % 100 = 0, @X, NULL)
	from #TempUID
go

select 
    index_id, partition_number, alloc_unit_type_desc
    ,index_level, page_count, page_count * 8 / 1024 as [Size MB]
	, avg_page_space_used_in_percent
from 
    sys.dm_db_index_physical_stats
    (
        db_id() /*Database */
        ,object_id(N'dbo.Customers') /* Table (Object_ID) */
        ,1 /* Index ID */
        ,null /* Partition ID  NULL  all partitions */
        ,'detailed' /* Mode */
    )
go
